;Version 0.3.1 build 2022-06-13 beta
#include-once
#include <GDIPlus.au3>

Enum $WEBP_PRESET_DEFAULT = 0,  _		; default preset.
		  $WEBP_PRESET_PICTURE, _     	; digital picture, like portrait, inner shot
		  $WEBP_PRESET_PHOTO,  _     	; outdoor photograph, with natural lighting
		  $WEBP_PRESET_DRAWING, _    	; hand or line drawing, with high-contrast details
		  $WEBP_PRESET_ICON, _        	; small-sized colorful images
		  $WEBP_PRESET_TEXT          	; text-like

; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_Ver
; Description ...:	Displays the DLL version information in a messagebox window
; Syntax ........:	WebP_Ver([$sPath2DLL = ""])
; Parameters ....:	$sPath2DLL			- [optional] a string value. Default is "". Path to WebP dll if not in script dir
; Return values .:	None
; Author ........:	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:	No
; ===============================================================================================================================
Func WebP_Ver($sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	DllCall($sDLL, "none", "WebP_DLL_Version")
	Return True
EndFunc   ;==>WebP_Ver

; #FUNCTION# ====================================================================================================================
; Name ..........: 	WebP_Ver2
; Description ...: 	Returns the DLL version information
; Syntax ........: 	WebP_Ver([$sPath2DLL = ""])
; Parameters ....: 	$sPath2DLL			- [optional] a string value. Default is "". Path to WebP dll if not in script dir
; Return values .: 	DLL version
; Author ........:	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......:	No
; ===============================================================================================================================
Func WebP_Ver2($sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	Return DllCall($sDLL, "str", "Web_DLL_Version2")[0]
EndFunc   ;==>WebP_Ver

; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_BitmapGetInfo
; Description ...:	Gets some rudimentary information about the WebP image
; Syntax ........:	WebP_BitmapGetInfo($sFilename[, $sPath2DLL = ""])
; Parameters ....:	$sFilename			- file to load
;                 	$sPath2DLL			- [optional] a string value. Default is "". Path to WebP dll if not in script dir
; Return values .:	Struct
; Author ........:	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:	https://developers.google.com/speed/webp
; Example .......:	No
; ===============================================================================================================================
Func WebP_BitmapGetInfo($sFilename, $sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	If Not FileExists($sFilename) Then Return SetError(2, 0, 0) ;file not found

	Local $iFileSize = FileGetSize($sFilename), $nBytes
	Local $tBuffer = DllStructCreate("struct;byte bin[" & $iFileSize & "];endstruct")
	Local Const $hFile = _WinAPI_CreateFile($sFilename, 2, 2)
	_WinAPI_ReadFile($hFile, $tBuffer, $iFileSize, $nBytes)
	_WinAPI_CloseHandle($hFile)

	If Int(BinaryMid($tBuffer.bin, 1, 4)) <> 1179011410 Or Int(BinaryMid($tBuffer.bin, 9, 6)) <> 88331643929943 Then Return SetError(3, 0, 0) ;header must contain RIFF and WEBPVP

	Local $tWebPBitstreamFeatures = DllStructCreate("struct;long width; long height; long has_alpha; long has_animation; long format; ulong pad[5];endstruct")
	Local $iReturn = DllCall($sDLL, "long", "WebP_BitmapGetInfo", "struct*", $tBuffer, "uint", $iFileSize, "struct*", $tWebPBitstreamFeatures)
	If $iReturn = 0 Then Return SetError(4, 0, 0)
	Return $tWebPBitstreamFeatures
EndFunc   ;==>WebP_BitmapGetInfo

; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_BitmapCreateGDIp
; Description ...:	Converts (decodes) a WebP image from disk to a GDI / GDI+ bitmap handle
; Syntax ........:	WebP_BitmasFilename[, $bGDIImage = False[, $sPath2DLL = ""]])
; Parameters ....:	$sFilename				- file to load
;                 	$bGDIImage				- [optional] a boolean value. Default is False (GDIPlus bitmap handle). If True then output is GDI bitmap handle
;                 	$bCountColors			- [optional] a boolean value. Default is False. If True then the colors will be counted and saved in extended. Use @extended to get color count.
;                 	$sPath2DLL				- [optional] a string value. Default is "". Path to WebP dll if not in script dir
; Return values .:	GDI / GDIPlus bitmap handle and color count if $bCountColors = True in extended.
; Author ........:	UEZ
; Modified ......:
; Remarks .......:	Currently only WebP images are supported - no animated WebP images yet!
; Related .......:
; Link ..........:		https://developers.google.com/speed/webp
; Example .......:	No
; ===============================================================================================================================
Func WebP_BitmapCreateGDIp($sFilename, $bGDIImage = False, $bCountColors = False, $sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	If Not FileExists($sFilename) Then Return SetError(2, 0, 0) ;file not found

	Local $iFileSize = FileGetSize($sFilename), $nBytes
	Local $tBuffer = DllStructCreate("byte bin[" & $iFileSize & "]")
	Local Const $hFile = _WinAPI_CreateFile($sFilename, 2, 2)
	_WinAPI_ReadFile($hFile, $tBuffer, $iFileSize, $nBytes)
	_WinAPI_CloseHandle($hFile)

	If Int(BinaryMid($tBuffer.bin, 1, 4)) <> 1179011410 Or Int(BinaryMid($tBuffer.bin, 9, 6)) <> 88331643929943 Then Return SetError(3, 0, 0) ;header must contain RIFF and WEBPVP
	Local $tColors = DllStructCreate("struct;ulong cc;endstruct")
	Local Const $hBitmap = DllCall($sDLL, "ptr", "WebP_BitmapCreateGDIp", "struct*", $tBuffer, "uint", $iFileSize, "boolean", $bGDIImage, "boolean", $bCountColors, "struct*", $tColors)[0]
	If $hBitmap = 0 Then Return SetError(4, 0, 0)

	Return SetExtended($tColors.cc, $hBitmap)
EndFunc   ;==>WebP_BitmapCreateGDIp

; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_BitmapCreateGDIpFromMem
; Description ...:	Converts (decodes) a WebP image from memory to a GDI / GDI+ bitmap handle
; Syntax ........:	WebP_BitmapCreateGDIpFromMem($tBuffer[, $iBufferSize = 0[, $bGDIImage = False[, $sPath2DLL = ""]]])
; Parameters ....:	$tBuffer						- a dll struct with WebP binary data as content or pointer to the memory data
;                 	$iBufferSize				- the size of the binary data (file size)
;                 	$bGDIImage				- [optional] a boolean value. Default is False (GDIPlus bitmap handle). If True then output is GDI bitmap handle
;                 	$bCountColors			- [optional] a boolean value. Default is False. If True then the colors will be counted and saved in extended. Use @extended to get color count.
;                 	$sPath2DLL				- [optional] a string value. Default is "". Path to WebP dll if not in script dir
; Return values .:	GDI / GDIPlus bitmap handle and color count if $bCountColors = True in extended.
; Author ........:	UEZ
; Modified ......:
; Remarks .......:	Currently only WebP images are supported - no animated WebP images yet!
; Related .......:
; Link ..........:	https://developers.google.com/speed/webp
; Example .......:	No
; ===============================================================================================================================
Func WebP_BitmapCreateGDIpFromMem($tBuffer, $iBufferSize = 0, $bGDIImage = False, $bCountColors = False, $sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	If $iBufferSize = 0 Then Return SetError(2, 0, 0)

	Local $binMem
	If IsPtr($tBuffer) Then
		Local $tMem = DllStructCreate("byte bin[" & $iBufferSize & "]", $tBuffer)
		$binMem = $tMem.bin
	Else
		$binMem = DllStructGetData($tBuffer, 1)
	EndIf
	If Int(BinaryMid($binMem, 1, 4)) <> 1179011410 Or Int(BinaryMid($binMem, 9, 6)) <> 88331643929943 Then Return SetError(3, 0, 0)  ;header must contain RIFF and WEBPVP

	Local $tColors = DllStructCreate("struct;ulong cc;endstruct")
	Local Const $hBitmap = DllCall($sDLL, "ptr", "WebP_BitmapCreateGDIp", "struct*", $tBuffer, "uint", $iBufferSize, "boolean", $bGDIImage, "boolean", $bCountColors, "struct*", $tColors)[0]
	If $hBitmap = 0 Then Return SetError(4, 0, 0)

	Return SetExtended($tColors.cc, $hBitmap)
EndFunc   ;==>WebP_BitmapCreateGDIpFromMem

; #FUNCTION# ====================================================================================================================
; Name ..........: 	WebP_CreateWebPLossySimpleFromBitmap
; Description ...: 	Converts a bitmap to WebP lossy image and save it to HD
; Syntax ........: 	WebP_CreateWebPLossySimpleFromBitmap($sFilename, $hBitmap[, $iQuality = 75[, $sPath2DLL = ""]])
; Parameters ....: 	$sFilename         	- file to load
;                  	$hBitmap            	- GDIPlus bitmap handle
;                  	$iQuality            	- [optional] an integer value. Default is 75. Valid range is 0 - 100.
;                  	$sPath2DLL         	- [optional] a string value. Default is "". Path to WebP dll if not in script dir.
; Return values .: 	0 for failure, 1 for success.
; Author ........: 	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........: 	https://developers.google.com/speed/webp
; Example .......: 	No
; ===============================================================================================================================
Func WebP_CreateWebPLossySimpleFromBitmap($sFilename, $hBitmap, $iQuality = 75, $sPath2DLL = "")
	If $sFilename = "" Then Return SetError(1, 0, 0)
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(2, 0, 0) ;DLL not found
	Local $iReturn = DllCall($sDLL, "long", "WebP_CreateWebPLossySimpleFromBitmap", "str", $sFilename, "ptr", $hBitmap, "float", $iQuality)[0]
	If $iReturn = 0 Then Return SetError(3, 0, 0)
	Return 1
EndFunc   ;==>WebP_CreateWebPLossySimpleFromBitmap

; #FUNCTION# ====================================================================================================================
; Name ..........: 	WebP_CreateWebPLosslessSimpleFromBitmap
; Description ...: 	Converts a bitmap to WebP lossless image and save it to HD
; Syntax ........: 	WebP_CreateWebPLosslessSimpleFromBitmap($sFilename, $hBitmap[, $sPath2DLL = ""])
; Parameters ....: 	$sFilename         	- file to load
;                  	$hBitmap            	- GDIPlus bitmap handle
;                  	$sPath2DLL         	- [optional] a string value. Default is "". Path to WebP dll if not in script dir.
; Return values .: 	0 for failure, 1 for success.
; Author ........: 	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........: 	https://developers.google.com/speed/webp
; Example .......: 	No
; ===============================================================================================================================
Func WebP_CreateWebPLosslessSimpleFromBitmap($sFilename, $hBitmap, $sPath2DLL = "")
	If $sFilename = "" Then Return SetError(1, 0, 0)
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(2, 0, 0) ;DLL not found
	Local $iReturn = DllCall($sDLL, "long", "WebP_CreateWebPLosslessSimpleFromBitmap", "str", $sFilename, "ptr", $hBitmap)[0]
	If $iReturn = 0 Then Return SetError(3, 0, 0)
	Return 1
EndFunc   ;==>WebP_CreateWebPLosslessSimpleFromBitmap


; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_CreateWebPAdvancedFromBitmap
; Description ...:	Converts a bitmap to WebP lossy / lossless image and save it to HD
; Syntax ........:	WebP_CreateWebPAdvancedFromBitmap($sFilename, $hBitmap[, $WebPPreset = $WEBP_PRESET_DEFAULT[, $lossless = 0[,
;                 									  $quality = 75.0[, $method = 4[, $sns_strength = 50[, $filter_sharpness = 0[, $filter_strength = 60[,
;														  $pass = 1[, $level = 6[, $sPath2DLL = ""]]]]]]]]]])
; Parameters ....:	$sFilename          		- file to load
;           			$hBitmap             		- GDIPlus bitmap handle
; 						$WebPPreset          		- [optional] an unknown value. Default is $WEBP_PRESET_DEFAULT.
; 						$lossless            		- [optional] an unknown value. Default is 0. 0 for lossy encoding / 1 for lossless..
; 						$quality             		- [optional] an unknown value. Default is 75.0. Valid range is 0 - 100.
; 						$method              		- [optional] a map. Default is 4. Valid range is 0 - 6 (0=fast, 6=slower-better).
; 						$sns_strength       		- [optional] a string value. Default is 50. Spatial Noise Shaping. 0=off, 100=maximum
; 						$filter_sharpness    		- [optional] a floating point value. Default is 0. Range: [0 = off .. 7 = least sharp]
; 						$filter_strength    		- [optional] a floating point value. Default is 60. Range: [0 = off .. 100 = strongest]
; 						$pass                		- [optional] a pointer value. Default is 1. Number of entropy-analysis passes (in [1..10]).
; 						$level              		- [optional] an unknown value. Default is 6. Between 0 (fastest, lowest compression) and 9 (slower, best compression) only valid for lossless = 1!
; 						$near_lossless				- [optional] an unknown value. Default is 100. Near lossless encoding [0 = max loss .. 100 = off (default)].
; 						$alpha_compression			- [optional] an unknown value. Default is 1. Algorithm for encoding the alpha plane (0 = none,1 = compressed with WebP lossless). Default is 1.
; 						$alpha_filtering			- [optional] an unknown value. Default is 1. Predictive filtering method for alpha plane.0: none, 1: fast, 2: best. Default if 1.
; 						$alpha_quality				- [optional] an unknown value. Default is 100. Between 0 (smallest size) and 100 (lossless). Default is 100.
; 						$target_size    			- [optional] an unknown value. Default is 0. If non-zero, set the desired target size in bytes.
; 						$NoSave     				- [optional] an unknown value. Default is False.
; 						$pMem			           	- [optional] a string value. Default is Null. If $NoSave = True then the pointer to the memory which holds the data will be returned.
;           			$pCallback         			- [optional] a pointer value. Default is 0. Pointer to a callback address for progress hook.
;           			$sPath2DLL         			- [optional] a string value. Default is "". Path to WebP dll if not in script dir.
; Return values .:	negative value for failure, 1 for success or the struct with information (pointers, size) if $NoSave = True
; Author ........:	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:	https://developers.google.com/speed/webp
; Example .......:	No
; ===============================================================================================================================
Func WebP_CreateWebPAdvancedFromBitmap($sFilename, $hBitmap, $WebPPreset = $WEBP_PRESET_DEFAULT, $lossless = 0, $quality = 75.0, $method = 4, $sns_strength = 50, _
																		$filter_sharpness = 0, $filter_strength = 60, $pass = 1, $level = 6, $near_lossless = 100, $alpha_compression = 1, $alpha_filtering = 1, $alpha_quality = 100, _
																		$target_size = 0, $NoSave = False, $pCallback = 0, $sPath2DLL = "")
	If $sFilename = ""  And Not $NoSave Then Return SetError(1, 0, 0)
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(2, 0, 0) ;DLL not found
	Local $tMem = DllStructCreate("struct;ptr pPic; ptr pWriter; ptr pMemData; uint memsize;endstruct")
	Local $iReturn = DllCall($sDLL, "long", "WebP_CreateWebPAdvancedFromBitmap", _
									  "str", $sFilename, _						;Webp filename
									  "ptr", $hBitmap, _						;handle to GDI+ bitmap
									  "long", $WebPPreset, _					;WebPPreset
									  "long", $lossless, _						;lossless
									  "float", $quality, _						;quality
									  "long", $method, _						;method
									  "long", $sns_strength, _					;sns_strength
									  "long", $filter_sharpness, _				;filter_sharpness
									  "long", $filter_strength, _				;filter_strength
									  "long", $pass, _							;pass
									  "long", $level, _							;level
									  "long", $near_lossless, _					;near_lossless
									  "long", $alpha_compression, _				;alpha_compression
									  "long", $alpha_filtering, _				;alpha_filtering
									  "long", $alpha_quality, _					;alpha_quality
									  "long", $target_size, _					;target_size
									  "bool", $NoSave, _						;
									  "struct*", $tMem, _						;
									  "ptr", @AutoItX64 ? $pCallback : 0)[0] 	;x86 crashes for unknown reason
	If $iReturn < 0 Then Return SetError(3, 0, $iReturn)
	If $NoSave And $tMem.memsize = 0 Then SetError(4, 0, 0)
	Return $NoSave ? $tMem : $iReturn
EndFunc   ;==>WebP_CreateWebPAdvancedFromBitmap

; #FUNCTION# ====================================================================================================================
; Name ..........:	WebP_FreeUp
; Description ...:	Release the ressources from $tMem struct
; Syntax ........:	WebP_FreeUp(Byref $tMem[, $sPath2DLL = ""])
; Parameters ....:	$tMem                	- [in/out] a dll struct value.
;                 	$sPath2DLL          - [optional] a string value. Default is "".
; Return values .:	1
; Author ........:	UEZ
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:	https://developers.google.com/speed/webp
; Example .......:	No
; ===============================================================================================================================
Func WebP_FreeUp(ByRef $tMem, $sPath2DLL = "")
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(1, 0, 0) ;DLL not found
	Local $iReturn = DllCall($sDLL, "long", "WebP_FreeUp", "struct*", $tMem)[0]
	Return $iReturn
EndFunc   ;==>WebP_FreeUp

; #FUNCTION# ====================================================================================================================
; Name ..........:	BitmapCountColors
; Description ...:	Counts the colors used by the bitmap
; Syntax ........:	BitmapCountColors($hBitmap)
; Parameters ....:	$hBitmap             - a handle to a GDI+ bitmap.
;                 	$bGDIImage			- [optional] a boolean value. Default is False (GDIPlus bitmap handle).
; Return values .:	Number of colors used by the image.
; Author ........:	UEZ
; Modified ......:
; Remarks .......:	The result may differ from other programs for JPG images depending on the decoder.
; Related .......:
; Link ..........:
; Example .......:	No
; ===============================================================================================================================
Func BitmapCountColors($hBitmap = 0, $bGDIImage = False, $sPath2DLL = "")
	If IsPtr($hBitmap) = 0 Or $hBitmap = 0 Then SetError(1, 0, 0)
	Local $sDLL = Path2DLL($sPath2DLL)
	If Not FileExists($sDLL) Then Return SetError(2, 0, 0) ;DLL not found
	Local $iReturn = DllCall(Path2DLL(), "ulong", "BitmapCountColors", "ptr", $hBitmap)[0]
	Return $iReturn
EndFunc

; #INTERNAL_USE_ONLY#============================================================================================================
; Name...........: 	Path2DLL
; Description ...: 	Return the path to the _WebP_x??.dll
; Author ........:	UEZ
; Modified.......:
; Remarks .......: 	This function is used internally by WebP.au3
; ===============================================================================================================================
Func Path2DLL($sPath2DLL = "")
	Return $sPath2DLL ? $sPath2DLL : @ScriptDir & (@AutoItX64 ? "\_WebP_x64.dll" : "\_WebP_x86.dll")
EndFunc   ;==>Path2DLL
